@@ -5,7 +5,7 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.14_03';
+our $VERSION = '0.14_04';
 $VERSION =~ tr/_//d;
 
 BEGIN {
@@ -29,20 +29,22 @@ sub _longmess {
   &Carp::longmess;
 }
 
+my @HOOKS = qw(__DIE__ __WARN__);
 my %OLD_SIG;
 
 sub import {
-  return if %OLD_SIG;
-  @OLD_SIG{qw(__DIE__ __WARN__)} = @SIG{qw(__DIE__ __WARN__)};
+  my $class = shift;
+  return if $OLD_SIG{$class};
+  @{ $OLD_SIG{$class} }{@HOOKS} = @SIG{@HOOKS};
 
-  $SIG{__DIE__}  = \&_die;
-  $SIG{__WARN__} = \&_warn;
+  @SIG{@HOOKS} = ($class->can('_die'), $class->can('_warn'));
   $Carp::Verbose = 'verbose';    # makes carp() cluck and croak() confess
 }
 
 sub unimport {
-  return unless %OLD_SIG;
-  @SIG{qw(__DIE__ __WARN__)} = @OLD_SIG{qw(__DIE__ __WARN__)};
+  my $class = shift;
+  return unless $OLD_SIG{$class};
+  @SIG{@HOOKS} = @{ delete $OLD_SIG{$class} }{@HOOKS};
   undef $Carp::Verbose;
 }
 
